using FitConnect.Encryption;
using FitConnect.Interfaces;
using FitConnect.Models;
using FitConnect.Services;
using FitConnect.Services.Models;
using FitConnect.Services.Models.v1;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;

namespace FitConnect;

/// <summary>
///     Fluent API for the FitConnect.Subscriber
/// </summary>
public class FluentSubscriber : Subscriber,
    IFluentSubscriber,
    IFluentSubscriberWithSubmission {
    private readonly string _privateKeyDecryption;

    private OAuthAccessToken? _token;


    public FluentSubscriber(FitConnectEnvironment environment,
        IOAuthService oAuthService,
        ISubmissionService submissionService,
        IRouteService routeService,
        IDestinationService destinationService,
        ICasesService casesService,
        string privateKeyDecryption,
        string privateKeySigning,
        ILogger? logger = null) : base(environment, oAuthService, submissionService,
        routeService, destinationService, casesService, logger) {
        _privateKeyDecryption = privateKeyDecryption;
        _encryption = new FitEncryption(privateKeyDecryption, privateKeySigning,
            "", "", logger);
    }


    /// <summary>
    /// Receives the available submissions from the server
    /// </summary>
    /// <param name="destinationId"></param>
    /// <param name="skip"></param>
    /// <param name="take"></param>
    /// <returns></returns>
    public IEnumerable<SubmissionForPickupDto> GetAvailableSubmissions(string? destinationId = null,
        int skip = 0,
        int take = 100) {
        SubmissionService.Token ??=
            AuthenticateAsync(Owner.ClientId, Owner.ClientSecret).Result?.AccessToken;

        var submissionsResult = SubmissionService.ListSubmissions(destinationId, 0, 100).Result;

        // Creating a dictionary of destinationId to submissionIds from the REST API result
        return submissionsResult.Submissions;
    }


    /// <summary>
    ///     Receives a specific submission from the server.
    /// </summary>
    /// <param name="submissionId"></param>
    /// <returns></returns>
    public IFluentSubscriberWithSubmission RequestSubmission(string submissionId) {
        if (!IsAuthenticated) Authenticate(Owner.ClientId, Owner.ClientSecret);
        SubmissionService.Token ??= _token.AccessToken;

        var submission = (Submission)SubmissionService.GetSubmission(submissionId);
        var (metaDataString, _, metaHash) = _encryption.Decrypt(submission.EncryptedMetadata);
        Console.WriteLine(metaDataString);
        submission.Metadata =
            JsonConvert.DeserializeObject<FitConnect.Models.Api.Metadata.Metadata>(metaDataString);

        if (submission.EncryptedData != null) {
            var (dataString, _, dataHash) = _encryption.Decrypt(submission.EncryptedData);
            submission.Data = dataString;
        }

        Submission = submission;
        return this;
    }

    public void CompleteSubmission(string caseId, FinishSubmissionStatus status) {
        throw new NotImplementedException();
    }

    public Submission Submission { get; private set; }


    /// <summary>
    ///     Reading attachments for a submission.
    /// </summary>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    public IEnumerable<Attachment> GetAttachments() {
        // TODO add guard calls

        var attachments = new List<Attachment>();
        foreach (var id in Submission!.AttachmentIds) {
            var encryptedAttachment = SubmissionService.GetAttachment(Submission.Id, id);
            var (_, content, hash) = _encryption.Decrypt(encryptedAttachment);
            var attachmentMeta =
                Submission.Metadata.ContentStructure.Attachments.First(a => a.AttachmentId == id);

            attachments.Add(new Attachment(attachmentMeta, content));
        }

        Submission.Attachments = attachments;
        return attachments;
    }

    /// <summary>
    /// </summary>
    /// <param name="clientId"></param>
    /// <param name="clientSecret"></param>
    /// <param name="scope"></param>
    /// <returns></returns>
    private FluentSubscriber
        Authenticate(string clientId, string clientSecret, string? scope = null) {
        _token ??= AuthenticateAsync(clientId, clientSecret, scope).Result;

        return this;
    }
}
