﻿using Autofac;
using FitConnect.BaseClasses;
using FitConnect.Encryption;
using FitConnect.Models;
using FitConnect.Models.Api.Metadata;
using FitConnect.Services;
using FitConnect.Services.Models;
using FitConnect.Services.Models.v1;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Attachment = FitConnect.Models.Attachment;
using Data = FitConnect.Models.Data;
using Metadata = FitConnect.Models.Metadata;

namespace FitConnect;

public class Sender : FunctionalBaseClass {
    internal Sender(FitConnectEnvironment environment,
        IOAuthService oAuthService,
        ISubmissionService submissionService,
        IRouteService routeService,
        IDestinationService destinationService,
        ILogger? logger = null
    ) : base(environment,
        logger, oAuthService, submissionService, routeService, destinationService) {
    }

    public FitEncryption? Encryption { get; protected set; }

    public static Sender Create(FitConnectEnvironment environment,
        ILogger? logger = null) {
        var container = DiContainer.GetContainer(environment, logger);
        var sender = container.Resolve<Sender>();
        return sender;
    }

    /// <summary>
    ///     Check public keys
    /// </summary>
    /// <returns></returns>
    public async Task<bool> CheckPublicKeyAsync(string publicKey) {
        throw new NotImplementedException();
    }


    /// <summary>
    ///     Check public keys
    /// </summary>
    /// <returns></returns>
    public async Task<bool> CheckPublicKeyAsync() {
        throw new NotImplementedException();
    }


    /// <summary>
    ///     Encrypt attachments (Anhänge)
    /// </summary>
    /// <param name="publicKey"></param>
    /// <param name="attachment"></param>
    /// <returns></returns>
    public KeyValuePair<string, string> Encrypt(string publicKey, Attachment attachment) {
        var content = Encryption.Encrypt(attachment.Content, publicKey);
        return new KeyValuePair<string, string>(attachment.Id, content);
    }

    /// <summary>
    ///     Encrypt attachments (Anhänge)
    /// </summary>
    /// <param name="publicKey">Public key for encryption</param>
    /// <param name="attachments">List of attachments to encrypt</param>
    /// <returns></returns>
    public Dictionary<string, string> Encrypt(string publicKey,
        IEnumerable<Attachment> attachments) {
        return attachments.Select(a => Encrypt(publicKey, a))
            .ToDictionary(p => p.Key, p => p.Value);
    }

    /// <summary>
    ///     Create Metadata incl. Hash
    /// </summary>
    /// <param name="submission"></param>
    /// <returns></returns>
    public string CreateMetadata(Submission submission) {
        var data = new Models.Api.Metadata.Data {
            SubmissionSchema = new Fachdatenschema {
                SchemaUri = submission.ServiceType.Identifier,
                MimeType = "application/json"
            }
        };
        var contentStructure = new ContentStructure {
            Data = data,
            Attachments = submission.Attachments.Select(a =>
                new Models.Api.Metadata.Attachment {
                    Description = a.Description,
                    AttachmentId = a.Id,
                    //MimeType = a.MimeType,
                    Filename = a.Filename,
                    Hash = new AttachmentHash {
                        Type = HashType.Sha512,
                        Content = a.Hash
                    }
                }).ToList()
        };

        var metaData = new Metadata {
            ContentStructure = contentStructure,
        };
        return JsonConvert.SerializeObject(metaData);
    }

    /// <summary>
    ///     Retrieve the events for the submission
    /// </summary>
    /// <param name="caseId"></param>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    public async Task<List<SecurityEventToken>> GetStatusForSubmission(string caseId) {
        if (!IsAuthenticated) {
            _oAuthToken = await AuthenticateAsync(Owner.ClientId, Owner.ClientSecret, null);
            SubmissionService.Token = _oAuthToken.AccessToken;
        }

        var events = SubmissionService.GetStatusForSubmissionAsync(caseId).Result;
        return events.Select(e => new SecurityEventToken(e)).ToList();
    }

    /// <summary>
    ///     Uploading the encrypted data to the server
    /// </summary>
    /// <param name="submissionId">Submissions ID</param>
    /// <param name="encryptedAttachments">Encrypted attachments with id and content</param>
    /// <exception cref="NotImplementedException"></exception>
    public async Task<bool> UploadAttachmentsAsync(string submissionId,
        Dictionary<string, string> encryptedAttachments) {
        // TODO: Upload encrypted attachments to server
        try {
            foreach (var (id, content) in encryptedAttachments) {
                Logger?.LogInformation("Uploading attachment {id}, content {content}", id, content);
                SubmissionService.UploadAttachment(submissionId, id, content);
            }

            return true;
        }
        catch (Exception e) {
            Logger?.LogError("Error Uploading attachment {message}", e.Message);
            throw;
        }
    }

}
