using System.IdentityModel.Tokens.Jwt;
using System.Reflection;
using System.Security.Cryptography.X509Certificates;
using System.Text.Json;
using Autofac;
using FitConnect.BaseClasses;
using FitConnect.Encryption;
using FitConnect.Models;
using FitConnect.RestService;
using FitConnect.Services;
using Jose;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json.Linq;
using Newtonsoft.Json.Schema;

namespace FitConnect;

public enum FinishSubmissionStatus {
    Accepted,
    Rejected,
    Forwarded
}

public class Subscriber : FunctionalBaseClass {
    private readonly ICasesService _casesService;
    protected FitEncryption _encryption;

    internal Subscriber(FitConnectEnvironment environment,
        IOAuthService oAuthService,
        ISubmissionService submissionService,
        IRouteService routeService,
        IDestinationService destinationService,
        ICasesService casesService,
        ILogger? logger = null) : base(environment, logger, oAuthService, submissionService,
        routeService, destinationService) {
        _casesService = casesService;
    }

    public static Subscriber Create(FitConnectEnvironment environment,
        X509Certificate2 certificate,
        ILogger? logger = null) {
        return DiContainer.GetContainer(environment, logger).Resolve<Subscriber>();
    }


    /// <summary>
    ///     Returns the content of an embedded resource file.
    ///     <para>Implemented for the JSON schemas</para>
    /// </summary>
    /// <param name="filename">File name of the resource</param>
    /// <returns>Content of the resource file as string</returns>
    /// <exception cref="Exception"></exception>
    private async Task<string> GetContentFromEmbeddedResource(string filename) {
        var assembly = typeof(Subscriber).GetTypeInfo().Assembly;
        var resourceName = $"{assembly.GetName().Name}.{filename}";
        await using var stream = assembly.GetManifestResourceStream(resourceName);
        if (stream == null) throw new Exception($"Could not find embedded resource {resourceName}");

        using var reader = new StreamReader(stream);
        return await reader.ReadToEndAsync();
    }


    /// <summary>
    ///     Checks the validity of the given metadata against the schema.
    /// </summary>
    /// <param name="metaData">JSON meta data</param>
    /// <returns></returns>
    public async Task<bool> CheckMetadataAsync(string metaData) {
        var jsonScheme =
            JSchema.Parse(await GetContentFromEmbeddedResource("metadata.schema.json"));

        var meta = JObject.Parse(metaData);
        return meta.IsValid(jsonScheme);
    }


    /// <summary>
    ///     Check Hash from Metadata
    /// </summary>
    /// <param name="metaData">Metadata in JSON Format</param>
    /// <returns></returns>
    public async Task<bool> CheckHashAsync(string metaData, string hash) {
        throw new NotImplementedException();
    }


    /// <summary>
    ///     Create SecurityEventToken and signature
    /// </summary>
    /// <param name="data"></param>
    /// <param name="attachment"></param>
    /// <param name="privateKey"></param>
    /// <returns></returns>
    public async Task<SecurityEventToken> CreateSecurityEventTokenAsync(string data,
        string attachment, string privateKey) {
        throw new NotImplementedException();
    }

    /// <summary>
    ///     Get single submission from API
    /// </summary>
    /// <param name="id">The id of the submission</param>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    public async Task<Submission> GetSubmissionAsync(string id) {
        throw new NotImplementedException();
    }

    /// <summary>
    ///     Get available submissions from API
    /// </summary>
    /// <param name="destinationId"></param>
    /// <param name="offset"></param>
    /// <param name="limit"></param>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    public async Task<List<Submission>> GetSubmissionsAsync(string destinationId, int offset = 0,
        int limit = 100) {
        throw new NotImplementedException();
    }


    public void AcceptSubmission(string caseId, string submissionId, string destinationId) =>
        CompleteSubmission(caseId, submissionId, destinationId, FinishSubmissionStatus.Accepted);

    public void RejectSubmission(string caseId, string submissionId, string destinationId) =>
        CompleteSubmission(caseId, submissionId, destinationId, FinishSubmissionStatus.Rejected);

    public void ForwardSubmission(string caseId, string submissionId, string destinationId) =>
        CompleteSubmission(caseId, submissionId, destinationId,
            FinishSubmissionStatus.Forwarded);


    public void CompleteSubmission(string caseId, string submissionId, string destinationId,
        FinishSubmissionStatus status) {
        var eventName = GetEvent(status);

        var token =
            _encryption.CreateSecurityEventToken(submissionId, caseId, destinationId, eventName);

        var tokenString = JsonSerializer.Serialize(token);
        Logger.LogWarning(tokenString);
        _casesService.FinishSubmission(caseId, tokenString);
    }

    private string GetEvent(FinishSubmissionStatus state) =>
        state switch {
            FinishSubmissionStatus.Accepted =>
                "https://schema.fitko.de/fit-connect/events/accept-submission",
            FinishSubmissionStatus.Rejected =>
                "https://schema.fitko.de/fit-connect/events/reject-submission",
            FinishSubmissionStatus.Forwarded =>
                "https://schema.fitko.de/fit-connect/events/forward-submission",
            _ => throw new ArgumentException("Invalid state")
        };
}
